; $Id$
;###############################################################################
;
; NAME:
;  LAUNCH_OPAN
;
; PURPOSE:
;  Launcher for PAN module
;
; CATEGORY:
;  DAVE, Data Analysis, PAN, curve fitting
;
; Richard Tumanjong Azuah
; NIST Center for Neutron Research
; azuah@nist.gov; (301) 9755604
; May 2008
;
; LICENSE:
;  The software in this file is written by an employee of
;  National Institute of Standards and Technology
;  as part of the DAVE software project.
;
;  The DAVE software package is not subject to copyright protection
;  and is in the public domain. It should be considered as an
;  experimental neutron scattering data reduction, visualization, and
;  analysis system. As such, the authors assume no responsibility
;  whatsoever for its use, and make no guarantees, expressed or
;  implied, about its quality, reliability, or any other
;  characteristic. The use of certain trade names or commercial
;  products does not imply any endorsement of a particular product,
;  nor does it imply that the named product is necessarily the best
;  product for the stated purpose. We would appreciate acknowledgment
;  if the DAVE software is used of if the code in this file is
;  included in another product.
;
;###############################################################################
pro launch_opan, group_leader=group_leader, workDir=workDir, dataDir=dataDir, DAVETool=oDAVETool, _EXTRA=etc

if (~obj_valid(oDAVETool)) then return

; Retrieve selected datasets from Data Manager
DAVEPtrArr = oDaveTool->GetSelectedDataAsDAVEPtr(count=count)
DAVENamesArr = oDaveTool->GetSelectedDataNameTag()

if (count gt 0) then begin
    ; loop through davePtr datasets and launch a PAN instance for each valid dataset
    for i=0,count-1 do begin
        if (ptr_valid(davePtrArr[i])) then $
        
        ;GET THE TITLE HEREws
        ;help,daveptrarr
        ;print,davenamesarr
            o = obj_new('OPAN', group_leader=group_leader, davePtr=davePtrArr[i], $
                        workDir=workDir, dataDir=dataDir, davetool=oDaveTool,wintitle=davenamesarr[i])
    endfor
    
    ; The datasets are stored as heap variables and must be cleaned up!
    heap_free, davePtrArr
endif else begin
    ; launch PAN without any initial data
    o = obj_new('OPAN', group_leader=group_leader, workDir=workDir, dataDir=dataDir, davetool=oDaveTool)
endelse

end